require 'active_record'

include ActiveRecord::Tasks

class SeedLoader
  def initialize(seed_file)
    @seed_file = seed_file
  end

  def load_seed
    raise "Seed file '#{@seed_file}' does not exist" unless File.file?(@seed_file)
    load @seed_file
  end
end

root = File.expand_path '../../..', __FILE__

DatabaseTasks.tap do |t|
  Application.start(:mysql)

  t.env = Application.env.to_s
  t.database_configuration = Application["mysql.config"]
  t.db_dir = File.join(root, 'db')
  t.fixtures_path = File.join(root, 'test', 'fixtures')
  t.migrations_paths = [File.join(root, 'db','migrate')]
  t.seed_loader = SeedLoader.new(File.join(root, 'db', 'seeds.rb'))
  t.root = root
end
# For reference to db rake tasks:
# https://github.com/rails/rails/blob/master/activerecord/lib/active_record/railties/databases.rake
load "active_record/railties/databases.rake"

namespace :db do
  namespace :seed do
    desc "Load builtin role docs and permissions from db/seeds and create join"
    task :sync do
      require 'activerecord-import'
      require 'role_management/stores'
      require 'role_management/stores/role_documents_directory'
      require 'role_management/models'

      # Kafka Message
      require 'role_management/utils/event_publish'
      require 'cht-lib-fabric/services'
      require 'cht-lib-fabric/contexts'
      require 'cht-lib-fabric/token_validator'

      include ::RoleManagement::Messages
      include ::RoleManagement::Stores
      include ::RoleManagement::Models::Privileges

      # Initialize the service account client
      service_acct_client =
        ::Platform::Services::ServiceAccount.new(
          local_auth_uri: Application[:settings].local_auth_uri,
          service_account: Application[:settings].service_account,
          api_key: Application[:settings].api_key
        )

      # Obtain and decode the refresh token, extracting the "sub" value
      service_acct_token = service_acct_client.fetch_service_account(request: ::Platform::Contexts::Request.current)
      decoded_token_result = ::Platform::TokenValidator.decode_token(service_acct_token)
      subject = decoded_token_result.value![:sub]

      # Create the request context and Kafka Message Instance
      request_context = OpenStruct.new(
        current_customer: nil,
        current_user: subject,
        current_organization: nil
      )

      context = {
        request: request_context
      }

      role_document_message = ::RoleManagement::Messages::RoleDocument.new

      ::ActiveRecord::Import.require_adapter("mysql2")

      apply_perm_tuples = lambda do |rd, action_subj_tuples|
        perm_ids = Permission.where("(action, subject) IN \
                                  (#{(['(?)']*action_subj_tuples.size).join(', ')})",
                                  *action_subj_tuples)
                           .pluck(:id)
        rd_id_to_perm_id = perm_ids.map { |perm_id| [rd.id, perm_id] }.uniq
        PermissionsRoleDocument.import([:role_document_id, :permission_id],
          rd_id_to_perm_id, ignore: true
        )
      end

      ::Rake::Task['db:seed'].invoke
      RoleDocument.where(system_defined: true).each do |rd|
        case rd.name
        when "Administrator"
          apply_perm_tuples.call(rd, builtin_customer_admin_privileges)
          role_document_message.push(rd, context, ::RoleManagement::Messages::BaseMessage::UPDATE)
        when "Power User"
          apply_perm_tuples.call(rd, builtin_power_user_privileges)
        when "Standard User"
          apply_perm_tuples.call(rd, builtin_standard_user_privileges)
        when "Tanzu Mission Control"
          apply_perm_tuples.call(rd, builtin_tmc_user_privileges)
        end
      end
    end
  end

  namespace :schema do
    desc "invoke db:schema:load if schema doesn't exist"
    task :load_if_new do
      logger = Application['logger']
      if ActiveRecord::Base.connection.tables.include?("schema_migrations")
        logger.info "Skipping schema load. it already exists"
      else
        logger.info "loading initial schema"
        Rake::Task["db:schema:load"].invoke
      end
    end
  end
end
