# frozen_string_literal: true

namespace :eng31313 do
  namespace :migrate do
    namespace :remove_unused_privs_from_custom_docs do
      desc "Create inventory of custom role documents which contains missing classic privileges and optionally delete these permissions from custom role documents"
        task :list, [:delete] => :environment do |t, args|
          logger = Application['logger']
          require 'activerecord-import'
          require 'role_management/stores'
          require 'role_management/stores/role_documents_directory'
          require 'role_management/models'
          require 'csv'
          include ::RoleManagement::Stores

          delete = ActiveModel::Type::Boolean.new.cast(args[:delete]) || false
          custom_role_documents = 0
          custom_role_documents_map = {}
          delta_perms_ids = []
          delta_perms_map = {}

          permissions_delta = [
            { :subject => "customer", :action => "update"},
            { :subject => "customer", :action => "read"},
            { :subject => "customer", :action => "delete"},
            { :subject => "jira_account", :action => "create"},
            { :subject => "jira_account", :action => "delete"},
            { :subject => "jira_account", :action => "read"},
            { :subject => "jira_account", :action => "update"},
            { :subject => "gcp_compute_disk_usage_daily_report", :action => "export"},
            { :subject => "gcp_compute_disk_usage_daily_report", :action => "read"},
            { :subject => "gcp_compute_disk_usage_daily_report", :action => "save"},
            { :subject => "gcp_compute_disk_usage_daily_report", :action => "share"},
            { :subject => "gcp_compute_disk_usage_daily_report", :action => "subscribe"},
            { :subject => "gcp_compute_image_usage_daily_report", :action => "export"},
            { :subject => "gcp_compute_image_usage_daily_report", :action => "read"},
            { :subject => "gcp_compute_image_usage_daily_report", :action => "save"},
            { :subject => "gcp_compute_image_usage_daily_report", :action => "share"},
            { :subject => "gcp_compute_image_usage_daily_report", :action => "subscribe"},
            { :subject => "gcp_compute_instance_usage_daily_report", :action => "export"},
            { :subject => "gcp_compute_instance_usage_daily_report", :action => "read"},
            { :subject => "gcp_compute_instance_usage_daily_report", :action => "save"},
            { :subject => "gcp_compute_instance_usage_daily_report", :action => "share"},
            { :subject => "gcp_compute_instance_usage_daily_report", :action => "subscribe"},
            { :subject => "gcp_compute_machine_type_profile", :action => "export"},
            { :subject => "gcp_compute_machine_type_profile", :action => "read"},
            { :subject => "gcp_compute_machine_type_profile", :action => "save"},
            { :subject => "gcp_compute_machine_type_profile", :action => "subscribe"},
            { :subject => "gcp_compute_snapshot_usage_daily_report", :action => "export"},
            { :subject => "gcp_compute_snapshot_usage_daily_report", :action => "read"},
            { :subject => "gcp_compute_snapshot_usage_daily_report", :action => "save"},
            { :subject => "gcp_compute_snapshot_usage_daily_report", :action => "share"},
            { :subject => "gcp_compute_snapshot_usage_daily_report", :action => "subscribe"},
            { :subject => "gcp_instance_usage_report", :action => "export"},
            { :subject => "gcp_instance_usage_report", :action => "read"},
            { :subject => "gcp_instance_usage_report", :action => "save"},
            { :subject => "gcp_instance_usage_report", :action => "share"},
            { :subject => "gcp_instance_usage_report", :action => "subscribe"},
            { :subject => "vmc_csp_organization", :action => "export"},
            { :subject => "vmc_csp_organization", :action => "read"},
            { :subject => "vmc_csp_organization", :action => "save"},
            { :subject => "vmc_csp_organization", :action => "subscribe"},
            { :subject => "vmc_csp_organization", :action => "tag"},
            { :subject => "gcp_dataproc_cluster", :action => "tag"},
            { :subject => "gcp_gke_cluster", :action => "tag"},
            { :subject => "gcp_storage_bucket", :action => "tag"},
            { :subject => "oci_generic_asset", :action => "export"},
            { :subject => "oci_region", :action => "tag"},
            { :subject => "gcp_compute_project", :action => "scan"}
          ]

          logger.info("Getting permission ids of all permissions from permission_delta")
          ActiveRecord::Base.transaction do
            permissions_delta.each do |perm|
              permissions_ids = RoleManagement::Stores::Permission.where(subject: perm[:subject], action: perm[:action]).pluck(:id)
              if permissions_ids && !permissions_ids.empty?
                delta_perms_ids.concat permissions_ids
                delta_perms_map[permissions_ids[0]] = {subject: perm[:subject], action: perm[:action]}
              end
            end
          end

          ActiveRecord::Base.transaction do
            RoleDocument.where(system_defined: false).each do |role_document|
              role_doc_str = "[#{role_document.name}] - [#{role_document.id}]"
              role_doc_formatted = "[#{role_document.customer_crn}] - #{role_doc_str}"
              logger.info "Inspecting custom role document for customer [#{role_document.customer_crn}] - #{role_doc_str}"
              # Build set of permissions configured for that role document
              role_document_permission_ids = RoleManagement::Stores::PermissionsRoleDocument.where(role_document_id: role_document.id).pluck(:permission_id)
              # Find intersection between what our role document has and and delta of privileges we got between classic and flex admin
              permissions_to_remove = role_document_permission_ids.to_set.intersection(delta_perms_ids.to_set)
              # If there is a intersection, it means role document needs to be cleaned
              unless permissions_to_remove.empty?
                logger.error "Customer [#{role_document.customer_crn}] role document #{role_doc_str} refers to non-existant permission(s) [#{permissions_to_remove.to_json}]"
                if delete
                  logger.info "Cleaning permissions [#{permissions_to_remove}] from role document #{role_doc_str} for customer [#{role_document.customer_crn}]"
                  sql = <<-SQL
                    DELETE FROM `permissions_role_documents`
                    WHERE `permissions_role_documents`.`role_document_id` = #{role_document.id}
                    AND `permissions_role_documents`.`permission_id` IN (#{permissions_to_remove.to_a.join(",")})
                  SQL
                  ActiveRecord::Base.connection.execute(sql.chomp)
                end
                custom_role_documents += 1
                custom_role_documents_map[role_doc_formatted] = permissions_to_remove
              end
            end
          end
          logger.info "Found [#{custom_role_documents}] custom role documents with non-existant permissions, deleted [#{delete}]"
          CSV.open("/tmp/custom_role_docs.csv", "wb") do |csv|
            csv << ["[Customer_Tenant] - [role_document_name] - [role_document_id]","Subject","Action"]
            custom_role_documents_map.each do |role_doc_formatted,permissions_to_remove|
              permissions_to_remove.each do |permission_id|
                csv << [role_doc_formatted, delta_perms_map[permission_id][:subject], delta_perms_map[permission_id][:action]]
              end
            end
          end
        end
    end
  end
end