# frozen_string_literal: true

namespace :sync do
  desc "Remove non-existant permissions from the system"
  task :cull_dead_permissions, [:execute] => :environment do |t, args|
    require 'cht-lib-fabric/crns'
    logger = Application['logger']

    execute = ActiveModel::Type::Boolean.new.cast(args[:execute])

    include RoleManagement::Models::Privileges

    all_permissions = RoleManagement::Stores::Permission.all
    priv_set = Set.new(all_privileges)
    to_cull = all_permissions.reject{|p| priv_set.include?([p.action.to_sym, p.subject.to_sym])}
    
    logger.info "Found #{to_cull.count} permissions to be culled"
    puts "Found #{to_cull.count} permissions to be culled"
    to_cull.each{|p| logger.info "  #{p.action}_#{p.subject}"}
    to_cull.each{|p| puts "#{p.action}_#{p.subject}"}

    if execute
      RoleManagement::Stores::Permission.where(id: to_cull.map(&:id)).destroy_all
    else
      puts "Aborting due to dry-run."
      logger.info "Aborting due to dry-run."
    end
    
  end
 
  desc "Produce report of non-existant permissions in custom role documents and optionally clean them"
  task :report, [:clean] => :environment do |t, args|
    logger = Application['logger']
    require 'activerecord-import'
    require 'role_management/stores'
    require 'role_management/stores/role_documents_directory'
    require 'role_management/models'
    include ::RoleManagement::Stores

    clean = ActiveModel::Type::Boolean.new.cast(args[:clean]) || false

    custom_role_documents = 0

    ActiveRecord::Base.transaction do
      # We don't have a way to get a list of customers easily, so go over each custom role document
      RoleDocument.where(system_defined: false).each do |role_document|
        role_doc_str = "[#{role_document.name}] - [#{role_document.id}]"
        puts "Inspecting custom role document for customer [#{role_document.customer_crn}] - #{role_doc_str}"
        logger.info "Inspecting custom role document for customer [#{role_document.customer_crn}] - #{role_doc_str}"
        # Build set of permissions configured for that role document
        role_document_permission_ids = RoleManagement::Stores::PermissionsRoleDocument.where(role_document_id: role_document.id).pluck(:permission_id)
        # Find those permissions in the master list of permissions for the role document
        actual_permission_ids = RoleManagement::Stores::Permission.where(id: [role_document_permission_ids]).pluck(:id)
        # Find difference between what our role document has and what is actually in the system
        diff = role_document_permission_ids.reject{|p| actual_permission_ids.include?(p)}
        # If there is a difference, it means role document needs to be cleaned
        unless diff.empty?
          puts "Customer [#{role_document.customer_crn}] role document #{role_doc_str} refers to non-existant permission(s) [#{diff.to_json}]"
          logger.error "Customer [#{role_document.customer_crn}] role document #{role_doc_str} refers to non-existant permission(s) [#{diff.to_json}]"
          if clean
            # Clean role document permissions from current role documents
            puts "Cleaning permissions [#{diff.to_json}] from role document #{role_doc_str} for customer [#{role_document.customer_crn}]"
            logger.info "Cleaning permissions [#{diff.to_json}] from role document #{role_doc_str} for customer [#{role_document.customer_crn}]"
            # This emulates how the service does it, active record will generate a DELETE FROM based on the ones missing
            # We can't apparently use AR `destroy` or `destroy_all` because this table has no `id` column
            # We pass in the variables (which are not user definable)
            sql = <<-SQL
              DELETE FROM `permissions_role_documents`
              WHERE `permissions_role_documents`.`role_document_id` = #{role_document.id}
              AND `permissions_role_documents`.`permission_id` IN (#{diff.join(",")})
            SQL
            ActiveRecord::Base.connection.execute(sql.chomp)
          end
          custom_role_documents += 1
        end
      end
    end

    puts "Found [#{custom_role_documents}] custom role documents with non-existant permissions, cleaned [#{clean}]"
    logger.info "Found [#{custom_role_documents}] custom role documents with non-existant permissions, cleaned [#{clean}]"
  end
end
